@@ -36,6 +36,24 @@ extern field id_s;
 extern field id_y;
 extern field id_d;
 
+#ifdef NEKO_WINDOWS
+
+static struct tm *localtime_r( time_t *t, struct tm *r ) {
+	struct tm *r2 = localtime(t);
+	if( r2 == NULL ) return NULL;
+	*r = *r2;
+	return r;
+}
+
+static struct tm *gmtime_r( time_t *t, struct tm *r ) {
+	struct tm *r2 = gmtime(t);
+	if( r2 == NULL ) return NULL;
+	*r = *r2;
+	return r;
+}
+
+#endif
+
 /**
 	date_now : void -> 'int32
 	<doc>Return current date and time</doc>
@@ -101,17 +119,16 @@ static value date_new( value s ) {
 **/
 static value date_format( value o, value fmt ) {
 	char buf[128];
-	struct tm *t;
+	struct tm t;
 	time_t d;
 	val_check(o,int32);
 	if( val_is_null(fmt) )
 		fmt = alloc_string("%Y-%m-%d %H:%M:%S");
-	val_check(fmt,string);	
+	val_check(fmt,string);
 	d = val_int32(o);
-	t = localtime(&d);
-	if( t == NULL )
+	if( localtime_r(&d,&t) == NULL )
 		neko_error();
-	strftime(buf,127,val_string(fmt),t);
+	strftime(buf,127,val_string(fmt),&t);
 	return alloc_string(buf);
 }
 
@@ -120,20 +137,19 @@ static value date_format( value o, value fmt ) {
 	<doc>Change the time of a date. Return the modified date</doc>
 **/
 static value date_set_hour( value o, value h, value m, value s ) {
-	struct tm *t;
+	struct tm t;
 	time_t d;
 	val_check(o,int32);
 	val_check(h,int);
 	val_check(m,int);
 	val_check(s,int);
 	d = val_int32(o);
-	t = localtime(&d);
-	if( t == NULL )
+	if( localtime_r(&d,&t) == NULL )
 		neko_error();
-	t->tm_hour = val_int(h);
-	t->tm_min = val_int(m);
-	t->tm_sec = val_int(s);
-	d = mktime(t);
+	t.tm_hour = val_int(h);
+	t.tm_min = val_int(m);
+	t.tm_sec = val_int(s);
+	d = mktime(&t);
 	if( d == -1 )
 		neko_error();
 	return alloc_int32(d);
@@ -144,20 +160,19 @@ static value date_set_hour( value o, value h, value m, value s ) {
 	<doc>Change the day of a date. Return the modified date</doc>
 **/
 static value date_set_day( value o, value y, value m, value d ) {
-	struct tm *t;
+	struct tm t;
 	time_t date;
 	val_check(o,int32);
 	val_check(y,int);
 	val_check(m,int);
 	val_check(d,int);
 	date = val_int32(o);
-	t = localtime(&date);
-	if( t == NULL )
+	if( localtime_r(&date,&t) == NULL )
 		neko_error();
-	t->tm_year = val_int(y) - 1900;
-	t->tm_mon = val_int(m) - 1;
-	t->tm_mday = val_int(d);
-	date = mktime(t);
+	t.tm_year = val_int(y) - 1900;
+	t.tm_mon = val_int(m) - 1;
+	t.tm_mday = val_int(d);
+	date = mktime(&t);
 	if( date == -1 )
 		neko_error();
 	return alloc_int32(date);
@@ -169,17 +184,16 @@ static value date_set_day( value o, value y, value m, value d ) {
 **/
 static value date_get_day( value o ) {
 	value r;
-	struct tm *t;
+	struct tm t;
 	time_t d;
 	val_check(o,int32);
 	d = val_int32(o);
-	t = localtime(&d);
-	if( t == NULL )
+	if( localtime_r(&d,&t) == NULL )
 		neko_error();
 	r = alloc_object(NULL);
-	alloc_field(r,id_y,alloc_int(t->tm_year + 1900));
-	alloc_field(r,id_m,alloc_int(t->tm_mon + 1));
-	alloc_field(r,id_d,alloc_int(t->tm_mday));
+	alloc_field(r,id_y,alloc_int(t.tm_year + 1900));
+	alloc_field(r,id_m,alloc_int(t.tm_mon + 1));
+	alloc_field(r,id_d,alloc_int(t.tm_mday));
 	return r;
 }
 
@@ -189,20 +203,39 @@ static value date_get_day( value o ) {
 **/
 static value date_get_hour( value o ) {
 	value r;
-	struct tm *t;
+	struct tm t;
 	time_t d;
 	val_check(o,int32);
 	d = val_int32(o);
-	t = localtime(&d);
-	if( t == NULL )
+	if( localtime_r(&d,&t) == NULL )
 		neko_error();
 	r = alloc_object(NULL);
-	alloc_field(r,id_h,alloc_int(t->tm_hour));
-	alloc_field(r,id_m,alloc_int(t->tm_min));
-	alloc_field(r,id_s,alloc_int(t->tm_sec));
+	alloc_field(r,id_h,alloc_int(t.tm_hour));
+	alloc_field(r,id_m,alloc_int(t.tm_min));
+	alloc_field(r,id_s,alloc_int(t.tm_sec));
 	return r;
 }
 
+/**
+	date_get_tz : void -> int
+	<doc>Return the local Timezone (in seconds)</doc>
+**/
+static value date_get_tz() {
+	struct tm local;
+	struct tm gmt;
+	int diff;
+	time_t raw = time(NULL);
+	if( localtime_r(&raw, &local) == NULL || gmtime_r(&raw, &gmt) == NULL )
+		neko_error();
+	diff = (local.tm_hour - gmt.tm_hour) * 3600 + (local.tm_min - gmt.tm_min) * 60;
+	// adjust for different days/years
+	if( gmt.tm_year > local.tm_year || gmt.tm_yday > local.tm_yday )
+		diff -= 24 * 3600;
+	else if( gmt.tm_year < local.tm_year || gmt.tm_yday < local.tm_yday )
+		diff += 24 * 3600;
+	return alloc_int(diff);
+}
+
 DEFINE_PRIM(date_now,0);
 DEFINE_PRIM(date_new,1);
 DEFINE_PRIM(date_format,2);
@@ -210,5 +243,6 @@ DEFINE_PRIM(date_set_hour,4);
 DEFINE_PRIM(date_set_day,4);
 DEFINE_PRIM(date_get_hour,1);
 DEFINE_PRIM(date_get_day,1);
+DEFINE_PRIM(date_get_tz,0);
 
 /* ************************************************************************ */
