@@ -16,6 +16,7 @@
 /* ************************************************************************ */
 #include <string.h>
 #include <neko.h>
+#include <neko_vm.h>
 #ifdef NEKO_WINDOWS
 #	include <winsock2.h>
 #	define FDSIZE(n)	(sizeof(u_int) + (n) * sizeof(SOCKET))
@@ -29,6 +30,7 @@
 #	include <sys/socket.h>
 #	include <sys/time.h>
 #	include <netinet/in.h>
+#	include <netinet/tcp.h>
 #	include <arpa/inet.h>
 #	include <unistd.h>
 #	include <netdb.h>
@@ -46,6 +48,15 @@
 #	define MSG_NOSIGNAL 0
 #endif
 
+#define NRETRYS	20
+
+typedef struct {
+	SOCKET sock;
+	char *buf;
+	int size;
+	int ret;
+} sock_tmp;
+
 typedef struct {
 	int max;
 #	ifdef NEKO_WINDOWS
@@ -122,6 +133,13 @@ static value socket_new( value udp ) {
 		neko_error();
 #	ifdef NEKO_MAC
 	setsockopt(s,SOL_SOCKET,SO_NOSIGPIPE,NULL,0);
+#	endif
+#	ifdef NEKO_POSIX
+	// we don't want sockets to be inherited in case of exec
+	{
+		int old = fcntl(s,F_GETFD,0);
+		if( old >= 0 ) fcntl(s,F_SETFD,old|FD_CLOEXEC);
+	}
 #	endif
 	return alloc_abstract(k_socket,(value)(int_val)s);
 }
@@ -132,7 +150,10 @@ static value socket_new( value udp ) {
 **/
 static value socket_close( value o ) {
 	val_check_kind(o,k_socket);
-	closesocket(val_sock(o));
+	POSIX_LABEL(close_again);
+	if( closesocket(val_sock(o)) ) {
+		HANDLE_EINTR(close_again);
+	}
 	val_kind(o) = NULL;
 	return val_true;
 }
@@ -150,8 +171,11 @@ static value socket_send_char( value o, value v ) {
 	if( c < 0 || c > 255 )
 		neko_error();
 	cc = (unsigned char)c;
-	if( send(val_sock(o),&cc,1,MSG_NOSIGNAL) == SOCKET_ERROR )
+	POSIX_LABEL(send_char_again);
+	if( send(val_sock(o),&cc,1,MSG_NOSIGNAL) == SOCKET_ERROR ) {
+		HANDLE_EINTR(send_char_again);
 		return block_error();
+	}
 	return val_true;
 }
 
@@ -171,19 +195,28 @@ static value socket_send( value o, value data, value pos, value len ) {
 	dlen = val_strlen(data);
 	if( p < 0 || l < 0 || p > dlen || p + l > dlen )
 		neko_error();
+	POSIX_LABEL(send_again);
 	dlen = send(val_sock(o), val_string(data) + p , l, MSG_NOSIGNAL);
-	if( dlen == SOCKET_ERROR )
+	if( dlen == SOCKET_ERROR ) {
+		HANDLE_EINTR(send_again);
 		return block_error();
+	}
 	return alloc_int(dlen);
 }
 
+static void tmp_recv( void *_t ) {
+	sock_tmp *t = (sock_tmp*)_t;
+	t->ret = recv(t->sock,t->buf,t->size,MSG_NOSIGNAL);
+}
+
 /**
 	socket_recv : 'socket -> buf:string -> pos:int -> len:int -> int
 	<doc>Read up to [len] bytes from [buf] starting at [pos] from a connected socket.
 	Return the number of bytes readed.</doc>
 **/
 static value socket_recv( value o, value data, value pos, value len ) {
-	int p,l,dlen;
+	int p,l,dlen,ret;
+	int retry = 0;
 	val_check_kind(o,k_socket);
 	val_check(data,string);
 	val_check(pos,int);
@@ -193,10 +226,21 @@ static value socket_recv( value o, value data, value pos, value len ) {
 	dlen = val_strlen(data);
 	if( p < 0 || l < 0 || p > dlen || p + l > dlen )
 		neko_error();
-	dlen = recv(val_sock(o), val_string(data) + p , l, MSG_NOSIGNAL);
-	if( dlen == SOCKET_ERROR )
+	POSIX_LABEL(recv_again);
+	if( retry++ > NRETRYS ) {
+		sock_tmp t;
+		t.sock = val_sock(o);
+		t.buf = val_string(data) + p;
+		t.size = l;
+		neko_thread_blocking(tmp_recv,&t);
+		ret = t.ret;
+	} else
+		ret = recv(val_sock(o), val_string(data) + p , l, MSG_NOSIGNAL);
+	if( ret == SOCKET_ERROR ) {
+		HANDLE_EINTR(recv_again);
 		return block_error();
-	return alloc_int(dlen);
+	}
+	return alloc_int(ret);
 }
 
 /**
@@ -204,10 +248,26 @@ static value socket_recv( value o, value data, value pos, value len ) {
 	<doc>Read a single char from a connected socket.</doc>
 **/
 static value socket_recv_char( value o ) {
+	int ret;
+	int retry = 0;
 	unsigned char cc;
 	val_check_kind(o,k_socket);
-	if( recv(val_sock(o),&cc,1,MSG_NOSIGNAL) <= 0 )
+	POSIX_LABEL(recv_char_again);
+	if( retry++ > NRETRYS ) {
+		sock_tmp t;
+		t.sock = val_sock(o);
+		t.buf = (char*)&cc;
+		t.size = 1;
+		neko_thread_blocking(tmp_recv,&t);
+		ret = t.ret;
+	} else
+		ret = recv(val_sock(o),&cc,1,MSG_NOSIGNAL);
+	if( ret == SOCKET_ERROR ) {
+		HANDLE_EINTR(recv_char_again);
 		return block_error();
+	}
+	if( ret == 0 )
+		neko_error();
 	return alloc_int(cc);
 }
 
@@ -224,9 +284,12 @@ static value socket_write( value o, value data ) {
 	cdata = val_string(data);
 	datalen = val_strlen(data);
 	while( datalen > 0 ) {
+		POSIX_LABEL(write_again);
 		slen = send(val_sock(o),cdata,datalen,MSG_NOSIGNAL);
-		if( slen == SOCKET_ERROR )
+		if( slen == SOCKET_ERROR ) {
+			HANDLE_EINTR(write_again);
 			return block_error();
+		}
 		cdata += slen;
 		datalen -= slen;
 	}
@@ -247,9 +310,12 @@ static value socket_read( value o ) {
 	val_check_kind(o,k_socket);
 	b = alloc_buffer(NULL);
 	while( true ) {
+		POSIX_LABEL(read_again);
 		len = recv(val_sock(o),buf,256,MSG_NOSIGNAL);
-		if( len == SOCKET_ERROR )
+		if( len == SOCKET_ERROR ) {
+			HANDLE_EINTR(read_again);
 			return block_error();
+		}
 		if( len == 0 )
 			break;
 		buffer_append_sub(b,buf,len);
@@ -266,7 +332,15 @@ static value host_resolve( value host ) {
 	val_check(host,string);
 	ip = inet_addr(val_string(host));
 	if( ip == INADDR_NONE ) {
-		struct hostent *h = gethostbyname(val_string(host));
+		struct hostent *h;
+#	if defined(NEKO_WINDOWS) || defined(NEKO_MAC)
+		h = gethostbyname(val_string(host));
+#	else
+		struct hostent hbase;
+		char buf[1024];
+		int errcode;
+		gethostbyname_r(val_string(host),&hbase,buf,1024,&h,&errcode);
+#	endif
 		if( h == NULL )
 			neko_error();
 		ip = *((unsigned int*)h->h_addr);
@@ -294,7 +368,14 @@ static value host_reverse( value host ) {
 	unsigned int ip;
 	val_check(host,int32);
 	ip = val_int32(host);
+#	if defined(NEKO_WINDOWS) || defined(NEKO_MAC)
 	h = gethostbyaddr((char *)&ip,4,AF_INET);
+#	else
+	struct hostent htmp;
+	int errcode;
+	char buf[1024];
+	gethostbyaddr_r((char *)&ip,4,AF_INET,&htmp,buf,1024,&h,&errcode);
+#	endif
 	if( h == NULL )
 		neko_error();
 	return alloc_string( h->h_name );
@@ -399,27 +480,22 @@ static value socket_select( value rs, value ws, value es, value timeout ) {
 	fd_set rx, wx, ex;
 	fd_set *ra, *wa, *ea;
 	value r;
-	while( true ) {
-		ra = make_socket_array(rs,&rx,&n);
-		wa = make_socket_array(ws,&wx,&n);
-		ea = make_socket_array(es,&ex,&n);
-		if( ra == &INVALID || wa == &INVALID || ea == &INVALID )
-			neko_error();
-		if( val_is_null(timeout) )
-			tt = NULL;
-		else {
-			val_check(timeout,number);
-			tt = &tval;
-			init_timeval(val_number(timeout),tt);
-		}
-		if( select((int)(n+1),ra,wa,ea,tt) == SOCKET_ERROR ) {
-#			if defined(NEKO_POSIX)
-			if( errno == EINTR )
-				continue;
-#			endif
-			neko_error();
-		}
-		break;
+	POSIX_LABEL(select_again);
+	ra = make_socket_array(rs,&rx,&n);
+	wa = make_socket_array(ws,&wx,&n);
+	ea = make_socket_array(es,&ex,&n);
+	if( ra == &INVALID || wa == &INVALID || ea == &INVALID )
+		neko_error();
+	if( val_is_null(timeout) )
+		tt = NULL;
+	else {
+		val_check(timeout,number);
+		tt = &tval;
+		init_timeval(val_number(timeout),tt);
+	}
+	if( select((int)(n+1),ra,wa,ea,tt) == SOCKET_ERROR ) {
+		HANDLE_EINTR(select_again);
+		neko_error();
 	}
 	r = alloc_array(3);
 	val_array_ptr(r)[0] = make_array_result(rs,ra);
@@ -459,9 +535,12 @@ static value socket_accept( value o ) {
 	unsigned int addrlen = sizeof(addr);
 	SOCKET s;
 	val_check_kind(o,k_socket);
+	POSIX_LABEL(accept_again);
 	s = accept(val_sock(o),(struct sockaddr*)&addr,&addrlen);
-	if( s == INVALID_SOCKET )
+	if( s == INVALID_SOCKET ) {
+		HANDLE_EINTR(accept_again);
 		return block_error();
+	}
 	return alloc_abstract(k_socket,(value)(int_val)s);
 }
 
@@ -705,13 +784,10 @@ static value socket_poll_events( value pdata, value timeout ) {
 	val_check(timeout,number);
 	p = val_poll(pdata);
 	tot = p->rcount + p->wcount;
-	while( true ) {
-		if( poll(p->fds,tot,(int)(val_number(timeout) * 1000)) < 0 ) {
-			if( errno == EINTR )
-				continue;
-			neko_error();
-		}
-		break;
+	POSIX_LABEL(poll_events_again);
+	if( poll(p->fds,tot,(int)(val_number(timeout) * 1000)) < 0 ) {
+		HANDLE_EINTR(poll_events_again);
+		neko_error();
 	}
 	k = 0;
 	for(i=0;i<p->rcount;i++)
@@ -752,6 +828,22 @@ static value socket_poll( value socks, value pdata, value timeout ) {
 	return a;
 }
 
+/**
+	socket_set_fast_send : 'socket -> bool -> void
+	<doc>
+	Disable or enable to TCP_NODELAY flag for the socket
+	</doc>
+**/
+static value socket_set_fast_send( value s, value f ) {
+	int fast;
+	val_check_kind(s,k_socket);
+	val_check(f,bool);
+	fast = val_bool(f);
+	if( setsockopt(val_sock(s),IPPROTO_TCP,TCP_NODELAY,(char*)&fast,sizeof(fast)) )
+		neko_error();
+	return val_null;
+}
+
 DEFINE_PRIM(socket_init,0);
 DEFINE_PRIM(socket_new,1);
 DEFINE_PRIM(socket_send,4);
@@ -771,6 +863,7 @@ DEFINE_PRIM(socket_host,1);
 DEFINE_PRIM(socket_set_timeout,2);
 DEFINE_PRIM(socket_shutdown,3);
 DEFINE_PRIM(socket_set_blocking,2);
+DEFINE_PRIM(socket_set_fast_send,2);
 
 DEFINE_PRIM(socket_poll_alloc,1);
 DEFINE_PRIM(socket_poll,3);
