@@ -28,12 +28,14 @@
 #	include <direct.h>
 #	include <conio.h>
 #else
+#	include <errno.h>
 #	include <unistd.h>
 #	include <dirent.h>
 #	include <limits.h>
 #	include <termios.h>
 #	include <sys/time.h>
 #	include <sys/times.h>
+#	include <xlocale.h>
 #endif
 
 #ifdef NEKO_MAC
@@ -101,9 +103,17 @@ static value sys_sleep( value f ) {
 #ifdef NEKO_WINDOWS
 	Sleep((DWORD)(val_number(f) * 1000));
 #else
-	if( (int)val_number(f) > 0 )
-		sleep((int)val_number(f));
-	usleep( (int)((val_number(f) - (int)val_number(f)) * 1000000) );
+	{
+		struct timespec t;
+		struct timespec tmp;
+		t.tv_sec = (int)val_number(f);
+		t.tv_nsec = (int)((val_number(f) - t.tv_sec) * 1e9);
+		while( nanosleep(&t,&tmp) == -1 ) {
+			if( errno != EINTR )
+				neko_error();
+			t = tmp;
+		}
+	}
 #endif
 	return val_true;
 }
@@ -113,8 +123,23 @@ static value sys_sleep( value f ) {
 	<doc>Set the locale for LC_TIME, returns true on success</doc>
 **/
 static value set_time_locale( value l ) {
+#ifdef NEKO_POSIX
+	locale_t lc, old;
+	val_check(l,string);
+	lc = newlocale(LC_TIME_MASK,val_string(l),NULL);
+	if( lc == NULL ) return val_false;
+	old = uselocale(lc);
+	if( old == NULL ) {
+		freelocale(lc);
+		return val_false;
+	}
+	if( old != LC_GLOBAL_LOCALE )
+		freelocale(old);
+	return val_true;
+#else
 	val_check(l,string);
 	return alloc_bool(setlocale(LC_TIME,val_string(l)) != NULL);
+#endif
 }
 
 /**
