@@ -33,6 +33,8 @@
 
 extern value *neko_builtins;
 
+DEFINE_KIND(neko_k_kind);
+
 /**
 	<doc>
 		<h1>Builtins</h1>
@@ -187,8 +189,11 @@ static value builtin_string( value v ) {
 	<doc>Return an uninitialized string of size [n]</doc>
 **/
 static value builtin_smake( value l ) {
+	value v;
 	val_check(l,int);
-	return alloc_empty_string( val_int(l) );
+	v = alloc_empty_string( val_int(l) );
+	memset(val_string(v),0,val_int(l));
+	return v;
 }
 
 /**
@@ -367,7 +372,7 @@ static value builtin_objcall( value o, value f, value args ) {
 **/
 static value builtin_objfield( value o, value f ) {
 	val_check(f,int);
-	return alloc_bool( val_is_object(o) && otable_find(((vobject*)o)->table, val_int(f)) != NULL );
+	return alloc_bool( val_is_object(o) && otable_find(&((vobject*)o)->table, val_int(f)) != NULL );
 }
 
 /**
@@ -377,7 +382,7 @@ static value builtin_objfield( value o, value f ) {
 static value builtin_objremove( value o, value f ) {
 	val_check(o,object);
 	val_check(f,int);
-	return alloc_bool( otable_remove(((vobject*)o)->table,val_int(f)) );
+	return alloc_bool( otable_remove(&((vobject*)o)->table,val_int(f)) );
 }
 
 static void builtin_objfields_rec( value d, field id, void *a ) {
@@ -391,9 +396,9 @@ static void builtin_objfields_rec( value d, field id, void *a ) {
 static value builtin_objfields( value o ) {
 	value a;
 	value *aptr;
-	objtable t;
+	objtable *t;
 	val_check(o,object);
-	t = ((vobject*)o)->table;
+	t = &((vobject*)o)->table;
 	a = alloc_array(otable_count(t));
 	aptr = val_array_ptr(a);
 	otable_iter(t,builtin_objfields_rec,&aptr);
@@ -711,6 +716,26 @@ static value builtin_float( value f ) {
 	return val_null;
 }
 
+/** <doc><h2>Abstract Builtins</h2></doc> **/
+
+/**
+	$getkind : 'abstract -> 'kind
+	<doc>Returns the kind value of the abstract</doc>
+**/
+static value builtin_getkind( value v ) {
+	val_check(v,abstract);
+	return alloc_abstract(neko_k_kind,val_kind(v));
+}
+
+/**
+	$iskind : any -> 'kind -> bool
+	<doc>Tells if a value is of the given kind</doc>
+**/
+static value builtin_iskind( value v, value k ) {
+	val_check_kind(k,neko_k_kind);
+	return val_is_abstract(v) ? alloc_bool(val_kind(v) == (vkind)val_data(k)) : val_false;
+}
+
 /** <doc><h2>Hashtable Builtins</h2></doc> **/
 
 /**
@@ -1226,6 +1251,9 @@ void neko_init_builtins() {
 	BUILTIN(isinfinite,1);
 	BUILTIN(istrue,1);
 
+	BUILTIN(getkind,1);
+	BUILTIN(iskind,2);
+
 	BUILTIN(hnew,1);
 	BUILTIN(hget,3);
 	BUILTIN(hmem,3);
