@@ -17,6 +17,7 @@
 #include "vm.h"
 #include "neko_mod.h"
 #include "objtable.h"
+#include <stdlib.h>
 #include <string.h>
 #include <math.h>
 #include <stdio.h>
@@ -27,8 +28,8 @@
 #	define USE_MMAP
 #endif
 
-#define tmp_alloc		alloc_private
-#define tmp_free(ptr)
+#define tmp_alloc(size) malloc(size)
+#define tmp_free(ptr)	free(ptr)
 
 #if defined(NEKO_X86) && !defined(NEKO_MAC)
 #define JIT_ENABLE
@@ -281,10 +282,10 @@ enum IOperation {
 
 #define XShl_rr(r,src)			if( src != Ecx ) ERROR; shift_r(r,4)
 #define XShl_rc(r,n)			shift_c(r,n,4)
-#define XShr_rr(r,src)			if( src != Ecx ) ERROR; shift_r(r,5)
-#define XShr_rc(r,n)			shift_c(r,n,5)
-#define XSar_rr(r,src)			if( src != Ecx ) ERROR; shift_r(r,7)
-#define XSar_rc(r,n)			shift_c(r,n,7)
+#define XShr_rr(r,src)			if( src != Ecx ) ERROR; shift_r(r,7)
+#define XShr_rc(r,n)			shift_c(r,n,7)
+#define XUShr_rr(r,src)			if( src != Ecx ) ERROR; shift_r(r,5)
+#define XUShr_rc(r,n)			shift_c(r,n,5)
 
 #define XIMul_rr(dst,src)		B(0x0F); B(0xAF); MOD_RM(3,dst,src)
 #define XIDiv_r(r)				B(0xF7); MOD_RM(3,7,r)
@@ -581,6 +582,20 @@ static void debug_method_call( int line, int stack ) {
 }
 #endif
 
+#ifdef JIT_DEBUG
+
+static void val_print_2( value v ) {
+	val_print(alloc_string(" "));
+	val_print(v);
+}
+
+static void val_print_3( value v ) {
+	val_print_2(v);
+	val_print(alloc_string("\n"));
+}
+
+#endif
+
 static jit_ctx *jit_init_context( void *ptr, int size ) {
 	jit_ctx *c = (jit_ctx*)alloc(sizeof(jit_ctx));
 	c->size = size;
@@ -1041,8 +1056,8 @@ static void jit_number_op( jit_ctx *ctx, enum Operation op ) {
 	is_int(ACC,false,jnot_int1);
 	is_int(TMP,false,jnot_int2);
 
-	XSar_rc(ACC,1);
-	XSar_rc(TMP,1);
+	XShr_rc(ACC,1);
+	XShr_rc(TMP,1);
 
 	if( op != OP_DIV ) {
 		switch( op ) {
@@ -1077,7 +1092,7 @@ static void jit_number_op( jit_ctx *ctx, enum Operation op ) {
 	XCmp_rb(TMP2,VAL_FLOAT);
 
 	XJump(JNeq,jerr1);
-	XSar_rc(ACC,1);
+	XShr_rc(ACC,1);
 	XPush_r(ACC);
 	XFILd_i(Esp);
 	XAdd_rc(TMP,4);
@@ -1106,7 +1121,7 @@ static void jit_number_op( jit_ctx *ctx, enum Operation op ) {
 	PATCH_JUMP(jint);
 	XAdd_rc(ACC,4);
 	XFLd_i(ACC);
-	XSar_rc(TMP,1);
+	XShr_rc(TMP,1);
 	XPush_r(TMP);
 	XFILd_i(Esp);
 	stack_pop(Esp,1);
@@ -1212,11 +1227,11 @@ static void jit_int_op( jit_ctx *ctx, enum IOperation op ) {
 
 	is_int(ACC,false,jerr1);
 	XMov_rr(TMP,ACC);
-	XSar_rc(TMP,1);
+	XShr_rc(TMP,1);
 	XMov_rp(ACC,SP,FIELD(0));
 
 	is_int(ACC,false,jerr2);
-	XSar_rc(ACC,1);
+	XShr_rc(ACC,1);
 
 	switch( op ) {
 	case IOP_SHL:
@@ -1226,7 +1241,7 @@ static void jit_int_op( jit_ctx *ctx, enum IOperation op ) {
 		XShr_rr(ACC,TMP);
 		break;
 	case IOP_USHR:
-		XSar_rr(ACC,TMP);
+		XUShr_rr(ACC,TMP);
 		break;
 	case IOP_AND:
 		XAnd_rr(ACC,TMP);
@@ -1277,7 +1292,7 @@ static void jit_add( jit_ctx *ctx, int _ ) {
 	XAdd_rc(TMP,4);
 	XFLd_i(TMP);
 	stack_push(Esp,1);
-	XSar_rc(ACC,1);
+	XShr_rc(ACC,1);
 	XPush_r(ACC);
 	XFILd_i(Esp);
 	XFAddp();
@@ -1291,7 +1306,7 @@ static void jit_add( jit_ctx *ctx, int _ ) {
 	XJump(JNeq,jsp_object1);
 	stack_pad(3);
 	XPush_c(1);
-	XSar_rc(ACC,1);
+	XShr_rc(ACC,1);
 	XPush_r(ACC);
 	XPush_r(TMP);;
 	XPush_r(VM);
@@ -1306,7 +1321,7 @@ static void jit_add( jit_ctx *ctx, int _ ) {
 	// is_float(acc) && is_int(sp) -> push(val_float(acc)+val_int(sp))
 	is_int(TMP,false,jnext);
 	stack_push(Esp,1);
-	XSar_rc(TMP,1);
+	XShr_rc(TMP,1);
 	XPush_r(TMP);
 	XFILd_i(Esp);
 	XAdd_rc(ACC,4);
@@ -1346,7 +1361,7 @@ static void jit_add( jit_ctx *ctx, int _ ) {
 	is_int(TMP,false,jnext);
 	stack_pad(3);
 	XPush_c(0);
-	XSar_rc(TMP,1);
+	XShr_rc(TMP,1);
 	XPush_r(TMP);
 	XPush_r(ACC);
 	XPush_r(VM);
@@ -1386,7 +1401,10 @@ static void jit_add( jit_ctx *ctx, int _ ) {
 	XCmp_rb(TMP2,VAL_OBJECT);
 	XJump(JEq,joop2);
 	PATCH_JUMP(jnext);
+	XMov_rp(TMP2,Esp,FIELD(1+PAD_OPT(2)));
+	XPush_r(TMP2);
 	label(code->oop_r[OP_ADD]);
+	stack_pop(Esp,1);
 	END();
 
 	// is_other(acc) && !is_int(sp) && is_string(sp) -> BUF
@@ -1405,7 +1423,10 @@ static void jit_add( jit_ctx *ctx, int _ ) {
 	// object op
 	PATCH_JUMP(joop1);
 	PATCH_JUMP(joop2);
+	XMov_rp(TMP2,Esp,FIELD(1+PAD_OPT(2)));
+	XPush_r(TMP2);
 	label(code->oop[OP_ADD]);
+	stack_pop(Esp,1);
 	END();
 
 	// errors
@@ -1463,7 +1484,7 @@ static void jit_array_access( jit_ctx *ctx, int n ) {
 
 	XJump(JNeq,jnot_array);
 	if( n > 0 ) {
-		XShr_rc(TMP,3);
+		XUShr_rc(TMP,3);
 		XCmp_rc(TMP,n);
 		XJump(JLte,jbounds);
 	}
@@ -1621,7 +1642,7 @@ static void jit_object_op_gen( jit_ctx *ctx, enum Operation op, int right ) {
 	XCall_m(val_field);
 	XCmp_rc(ACC,CONST(val_null));
 	XJump(JNeq,next);
-	stack_pop(Esp,7);
+	stack_pop(Esp,(op == OP_SET)?7:6);
 	runtime_error(21,true); // Unsupported operation
 	PATCH_JUMP(next);
 	XPop_r(TMP);
@@ -1711,9 +1732,9 @@ static void jit_opcode( jit_ctx *ctx, enum OPCODE op, int p ) {
 		is_int(ACC,false,jerr2);
 
 		// check bounds & access array
-		XSar_rc(ACC,1);
+		XShr_rc(ACC,1);
 		XMov_rp(TMP2,TMP,FIELD(0));
-		XShr_rc(TMP2,3);
+		XUShr_rc(TMP2,3);
 		XCmp_rr(ACC,TMP2);
 		XJump(JGte,jbounds);
 		XAdd_rc(ACC,1);			  // acc = val_array_ptr(tmp)[acc]
@@ -1766,13 +1787,14 @@ static void jit_opcode( jit_ctx *ctx, enum OPCODE op, int p ) {
 		XMov_rr(VM,ACC);
 		XPush_c(p);
 		start = buf.c;
-		XMov_rp(TMP,VM,FIELD(1));
+		XMov_rr(TMP,VM);
+		XAdd_rc(TMP,FIELD(1));
 		XPush_r(TMP);
 		XCall_m(otable_find);
 		XCmp_rc(ACC,0);
 		XJump(JNeq,jend1);
 		stack_pop(Esp,1);
-		XMov_rp(VM,VM,FIELD(2)); // acc = acc->proto
+		XMov_rp(VM,VM,FIELD(3)); // acc = acc->proto
 		XCmp_rc(VM,0);
 		XJump(JNeq,loop);
 		*loop = (int)(start - buf.c);
@@ -1842,7 +1864,7 @@ static void jit_opcode( jit_ctx *ctx, enum OPCODE op, int p ) {
 		stack_pad(2);
 		XPush_r(ACC);
 		XPush_c(p);
-		XMov_rp(TMP,TMP,FIELD(1));
+		XAdd_rc(TMP,FIELD(1));
 		XPush_r(TMP);
 		XCall_m(otable_replace);
 		stack_pop(Esp,3);
@@ -1871,8 +1893,8 @@ static void jit_opcode( jit_ctx *ctx, enum OPCODE op, int p ) {
 		is_int(TMP2,false,jerr2);
 
 		XMov_rp(TMP,TMP,FIELD(0)); // tmp = tmp->type
-		XSar_rc(TMP2,1);
-		XShr_rc(TMP,3);
+		XShr_rc(TMP2,1);
+		XUShr_rc(TMP,3);
 		XCmp_rr(TMP2,TMP);
 		XJump(JGte,jend1);
 
@@ -2339,7 +2361,7 @@ static void jit_opcode( jit_ctx *ctx, enum OPCODE op, int p ) {
 
 		// trap = val_int(sp[5])
 		XMov_rp(TMP,SP,FIELD(5));
-		XSar_rc(TMP,1);
+		XShr_rc(TMP,1);
 		set_var_r(VTrap,TMP);
 		pop(6);
 		break;
@@ -2436,7 +2458,7 @@ static void jit_opcode( jit_ctx *ctx, enum OPCODE op, int p ) {
 		XJump_near(jnext2);
 		PATCH_JUMP(jok2);
 		XMov_rr(TMP2,ACC);
-		XSar_rc(TMP2,1);
+		XShr_rc(TMP2,1);
 		PATCH_JUMP(jnext1);
 		PATCH_JUMP(jnext2);
 		get_var_r(TMP,VModule);
@@ -2510,8 +2532,12 @@ static char *alloc_jit_mem( int size ) {
 	// round to next page
 	size += (4096 - size%4096);
 	p = (int*)mmap(NULL,size,PROT_READ|PROT_WRITE|PROT_EXEC,(MAP_PRIVATE|MAP_ANON),-1,0);
-	if( p == (int*)-1 )
-		val_throw(alloc_string("Failed to allocate JIT memory"));
+	if( p == (int*)-1 ) {
+		buffer b = alloc_buffer("Failed to allocate JIT memory ");
+		val_buffer(b,alloc_int(size>>10));
+		val_buffer(b,alloc_string("KB"));
+		val_throw(buffer_to_string(b));
+	}
 	*p = size;
 	return (char*)(p + 1);
 }
@@ -2636,7 +2662,7 @@ void neko_module_jit( neko_module *m ) {
 			stack_pop(Esp,1);
 			// val_print(pc_pos)
 			XPush_c(CONST(alloc_int(i)));
-			XCall_m_real(val_print);
+			XCall_m_real(val_print_2);
 			stack_pop(Esp,1);
 			// val_print(alloc_int(spmax - sp))
 			get_var_r(TMP,VSpMax);
@@ -2644,16 +2670,16 @@ void neko_module_jit( neko_module *m ) {
 			XShr_rc(TMP,1);
 			XOr_rc(TMP,1);
 			XPush_r(TMP);
-			XCall_m_real(val_print);
+			XCall_m_real(val_print_2);
 			stack_pop(Esp,1);
 			// val_print(alloc_int(csp - spmin))
 			XMov_rp(TMP2,VM,VMFIELD(spmin));
 			XMov_rr(TMP,CSP);
 			XSub_rr(TMP,TMP2);
-			XSar_rc(TMP,1);
+			XShr_rc(TMP,1);
 			XOr_rc(TMP,1);
 			XPush_r(TMP);
-			XCall_m_real(val_print);
+			XCall_m_real(val_print_3);
 			stack_pop(Esp,1);
 
 			XPop_r(ACC);
