@@ -18,47 +18,62 @@
 #define __OBJTABLE_H
 #include "neko.h"
 
-typedef struct {
-	field id;
-	value v;
-} cell;
+static INLINE void otable_init(objtable *t) {
+	t->count = 0;
+	t->cells = NULL;
+}
 
-struct _objtable
-{
-	int count;
-	cell *cells;
-};
-
-static INLINE value *otable_find(objtable t,field id) {
+static INLINE value *otable_find(objtable *t,field id) {
 	int min;
 	int max;
 	int mid;
+	objcell *c;
 	field cid;
-	if( !t->count )
-		return NULL;
-	max = t->count;
 	min = 0;
+	max = t->count;
+	c = t->cells;
 	while( min < max ) {
 		mid = (min + max) >> 1;
-		cid = t->cells[mid].id;
+		cid = c[mid].id;
 		if( cid < id )
 			min = mid + 1;
 		else if( cid > id )
 			max = mid;
 		else
-			return &t->cells[mid].v;
+			return &c[mid].v;
 	}
 	return NULL;
 }
 
-objtable otable_empty();
-void otable_replace(objtable t, field id, value data);
-int otable_remove(objtable t, field id);
-void otable_optimize(objtable t);
+static INLINE value otable_get(objtable *t,field id) {
+	int min;
+	int max;
+	int mid;
+	objcell *c;
+	field cid;
+	min = 0;
+	max = t->count;
+	c = t->cells;
+	while( min < max ) {
+		mid = (min + max) >> 1;
+		cid = c[mid].id;
+		if( cid < id )
+			min = mid + 1;
+		else if( cid > id )
+			max = mid;
+		else
+			return c[mid].v;
+	}
+	return val_null;
+}
+
+void otable_replace(objtable *t, field id, value data);
+int otable_remove(objtable *t, field id);
+void otable_optimize(objtable *t);
 #define otable_count(t)	(t)->count
 
-objtable otable_copy(objtable t);
-void otable_iter(objtable t, void f( value data, field id, void *), void *p );
+void otable_copy(objtable *t, objtable *target);
+void otable_iter(objtable *t, void f( value data, field id, void *), void *p );
 
 #endif
 /* ************************************************************************ */
